#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/4/27 11:35
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

import requests
import allure
from dg_itest import get_logger, host_url

class DgRequests(object):
	def __init__(self):
		self.server_url = host_url

	def http_request(self, method, url, headers=None, **kwargs):
		"""
		针对接口封装.
		"""
		method = method.upper()

		request_headers = {'finger': 'auto_test'}
		if headers is not None:
			request_headers.update(headers)
		allure.attach(str(kwargs))
		get_logger().info(f'inputs params: {str(kwargs)}')
		result = requests.request(method, url=self.server_url + url, headers=request_headers, **kwargs)
		get_logger().info(f'response: {str(result.json())}')
		allure.attach(str(result.json()))
		return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/21 17:02
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

from dg_itest.utils.replace import Replace

def test_replace():
	pattern = r'\$.*?\$'
	source = {'test': '123', 'key': {'key1': {'key1': ['$change1$', '$change2$']}, 'key2': ['change1', 'change2']}}
	update = {'$change1$': {'key': 'source_file.png', 'value': '/data/test1.png'}, '$change2$': {'key': 'target_file.png', 'value': '/data/test1.png'}}

	result = Replace.replace_dict(pattern, source, update)
	print(result)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/10/20 10:21
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

import difflib


class DiffHelper:
    """
    比较差异
    """
    @staticmethod
    def diff(source, target):
        if isinstance(source, str) and isinstance(target, str):
            differ = difflib.Differ()
            str_diff = differ.compare(source.splitlines(), target.splitlines())
            return '\n'.join(list(str_diff))

    def _diff_dict(self, source: dict, target: dict):
        """
        对比dict的差异
        """

        differ = set(source.items()) ^ set(target.items())
        return differ



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/19 19:01
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
from openpyxl import *


class XlsxHandler():
	def __init__(self, file_path):
		self.file_path = file_path

	def load(self):
		raw = []
		workbook = load_workbook(self.file_path)
		table = workbook.active
		for row in range(2, table.max_row + 1):
			tester = {"name": None, "request": {}, "validate": [], "expect": None}
			for col in range(table.max_column):
				col_title = table.cell(1, col + 1).value
				if col_title in ['url', 'method', 'params']:
					request = tester.get("request")
					if col_title == "params":
						request_params = eval(table.cell(row, col + 1).value)
						request.update({table.cell(1, col + 1).value: request_params})
					else:
						request.update({table.cell(1, col + 1).value: table.cell(row, col + 1).value})
					tester.update({"request": request})
				elif col_title == "validate":
					if table.cell(row, col + 1).value is not None and table.cell(row, col + 1).value != '':
						tester.get("validate").append(json.loads(table.cell(row, col + 1).value))
				elif col_title == "expect":
					tester.update({table.cell(1, col + 1).value: json.loads(table.cell(row, col + 1).value)})
				else:
					tester.update({table.cell(1, col + 1).value: table.cell(row, col + 1).value})
			if not ("status" in tester.keys() and tester.get("status") == 0) :
				raw.append({"test": tester})
		return raw



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/20 11:16
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

__all__ = ["DgItest"]

import os
import logging
import sys
from pathlib import *
from dg_itest.utils.env_generater import EnvGenerater

import pytest

_logger = logging.root

def set_logger(new_logger):
    global _logger
    _logger = new_logger


def get_logger():
    global _logger
    return _logger

host_url = ""
local_test_res = ""

class DgItest:
    '''
    接口自动化测试框架
    '''
    def __init__(self, url, test_src, test_res):
        '''
        url: 测试地址Host
        test_src: 测试用例归档目录
        test_res: 测试资源归档目录
        '''
        global host_url
        host_url = url
        global local_test_res
        local_test_res = test_res
        self.test_src = test_src

    def run(self):
        test_report = 'test_report/result/'

        if not Path(test_report).exists():
            Path(test_report).mkdir(parents=True)

        pytest.main(['-s', self.test_src, '--alluredir', Path(test_report).resolve().as_posix()])

        try:
            # 生成allure report中environment相关配置
            EnvGenerater.generate_env_file(host_url, Path(test_report))

            # 本地生成allure report 使用; 本地生成报告使用，commit时请注释
            os.system("allure generate ./test_report/result/ -o ./test_report/report --clean")
        except Exception:
            get_logger().error("请确保本地安装有allure，参考：https://docs.qameta.io/allure/")

if __name__ == '__main__':
    url = sys.argv[0]
    test_src = sys.argv[1]
    test_res = sys.argv[2]

    DgItest(url, test_src, test_res).run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/21 14:44
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
import re

class Replace:
	@staticmethod
	def replace(pattern: str, source, update : dict):
		if isinstance(source, str):
			result = Replace.replace_str(pattern, source, update)
		elif isinstance(source, dict):
			result = Replace.replace_dict(pattern, source, update)
		else:
			result = source
		return result


	@classmethod
	def replace_str(cls, pattern: str, source: str, update: dict) -> str:
		re_pattern = re.compile(pattern, re.DOTALL)
		match_results = re_pattern.findall(source)

		for item in match_results:
			source = source.replace(item, update.get(item))
		return source

	@classmethod
	def replace_dict(cls, pattern: str, source: dict, update) -> dict:
		re_pattern = re.compile(pattern, re.DOTALL)
		match_results = re_pattern.findall(json.dumps(source))
		result = {}
		for match_item in match_results:
			result = replace(source, match_item, update.get(match_item))
		return result


def replace(source: dict, match_item: str, update) -> dict:
	for k, v in source.items():
		if isinstance(v, dict):
			source[k] = replace(v, match_item, update)
		elif isinstance(v, list):
			if match_item in v:
				match_index = v.index(match_item)
				v[match_index] = update
				source.update({k: v})
		elif isinstance(v, str):
			if v == match_item:
				source.update({k: v})
			elif v.__contains__(match_item):
				replace_v = v.replace(match_item, str(update))
				source.update({k: replace_v})
			else:
				pass
		else:
			pass

	return source


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/19 18:25
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
import re
import pytest
import jsonpath
from pathlib import *
from dg_itest import local_test_res
from dg_itest.utils.diff_helper import DiffHelper
from dg_itest.servers.dg_servers.dg_singleton import DgSingleton
from dg_itest.utils.cache import local_cache
from dg_itest.utils.replace import Replace


class TestItem(pytest.Item):
    def __init__(self, name, parent, values):
        super(TestItem, self).__init__(name, parent)
        self.name = name
        self.values = values
        self.request = self.values.get("request")
        self.validate = self.values.get("validate")
        self.expect = self.values.get('expect')

    def runtest(self):
        request_data = self.values['request']
        params = self.replace(request_data)
        params = request_data['params']
        if "files" in params.keys():
                params.update({"files": self.get_files(params.get("files"))})
        request_data.pop('params')
        request_data.update(params)

        try:
            api = DgSingleton().apis
            response = api.http_request(**request_data)
        except Exception as ex:
            print("request Error！\n" + str(ex))

        self.assert_response(response)

    # todo 断言类型需要增加，目前只支持eq(相等), contains(包含)操作判断。
    def assert_response(self, response):
        for item in self.validate:
            if "eq" in item.keys():
                validate_rule = item.get("eq")
                actual_result = jsonpath.jsonpath(response.json(), validate_rule)
                expect_result = jsonpath.jsonpath(self.expect.get('json'), validate_rule)

                if isinstance(actual_result, list) and isinstance(expect_result, list):
                    actual_result = sorted(actual_result)
                    expect_result = sorted(expect_result)
                assert actual_result == expect_result, '\n' + DiffHelper.diff(str(actual_result), str(expect_result))
            if "sa" in item.keys():
                sa_value = item.get("sa")
                for sa_item_key in sa_value.keys():
                    sa_item_value =  jsonpath.jsonpath(response.json(), sa_value.get(sa_item_key))
                    assert type(sa_item_value) is list and len(sa_item_value) > 0, '\n' + '未获取到值'
                    local_cache.update({f"${sa_item_key}$": sa_item_value[0]})

    def replace(self, source):
        pattern_str = r'\$.*?\$'
        source_str = json.dumps(source)
        pattern = re.compile(pattern_str, re.DOTALL)
        match_items = pattern.findall(source_str)
        update = {}
        for items in match_items:
            update.update({items: local_cache[items]})
        if len(update.keys()) > 0:
            result = Replace.replace(pattern_str, source, update)
            return result
        else:
            return source

    def get_files(self, files_array):
        all_resource_files = Path(local_test_res).rglob("*.*")
        files_buffer = []
        for file_name in files_array:
            file = [file_item for file_item in all_resource_files if file_item.name == file_name]
            if len(file) > 0:
                suffix = file[0].suffix
                if suffix in ['.jpg', '.jpeg', '.png']:
                    content_type = f'image/{suffix.lstrip(".")}'
                else:
                    content_type = f'application/{suffix.lstrip(".")}'

                files_buffer.append(('file', (file_name, open(file[0].resolve(), 'rb'), content_type)))
            else:
                continue
        return files_buffer




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/20 11:17
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

import os
from setuptools import setup, find_packages

def get_requirements():
    requirements = 'requirements.txt'
    if not os.path.exists(requirements):  # install from tar
        requirements = 'dg_itest.egg-info/requires.txt'
    with open(requirements) as fp:
        install_requires = fp.read()
    return install_requires.split('\n')

setup(name='dg-itest',
      version='0.0.8',
      description='接口自动化测试框架',
      url='https://github.com/yaitza/dg-itest',
      download_url='https://github.com/yaitza/dg-itest/releases',
      author='yaitza',
      author_email='yaitza@foxmail.com',
      packages=find_packages(),
      package_data={'': ['*.py']},
      install_requires=get_requirements(),
      zip_safe=False)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/19 18:55
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import traceback
from pathlib import Path
from .yaml_handler import YamlHandler
from .xlsx_handler import XlsxHandler

class FileHandler:
	def __init__(self, file_path):
		self.file_path = file_path

	def load(self):
		try:
			file = Path(self.file_path)
			if file.suffix == '.yml':
				return YamlHandler(self.file_path).load()
			if file.suffix in [".xls", ".xlsx"]:
				return XlsxHandler(self.file_path).load()
		except Exception:
			traceback.print_exc()






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/10/12 17:55
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

import pytest
from dg_itest.servers.file_handler.file_handler import FileHandler
from dg_itest.servers.test_handler.test_item import TestItem

class TestFile(pytest.File):
    def collect(self):
        raw = FileHandler(self.fspath).load()
        for test_case in raw:
            name = test_case['test']['name']
            values = test_case['test']
            yield TestItem.from_parent(self, name=name, values=values)



